==> pvfs-1.6.1-pre1.tgz: pvfs-1.6.1-pre1/lib/pvfs_symlink.c <==
/*
 * (C) 1995-2001 Clemson University and Argonne National Laboratory.
 *
 * See COPYING in top-level directory.
 */


/* This file contains PVFS library call for unlink.   	*/
/* It will determine if file is UNIX or PVFS and then 	*/
/* make the proper request.										*/

#include <lib.h>
#include <sys/param.h>
#include <meta.h>
#include <errno.h>

/* This file contains PVFS library call for symlink and hard links.   	*/
/* It will determine if file is UNIX or PVFS and then 						*/
/* make the proper request.															*/


extern int pvfs_checks_disabled;

int pvfs_symlink(const char* target_name, const char *link_name)
{
	int i;
	mreq req;
	mack ack;
	struct sockaddr *saddr;
	char *fn;
	int64_t fs_ino;
	char bothname[MAXPATHLEN + MAXPATHLEN + 2];
	int len = 0, bothlen = 0;

	memset(&req, 0, sizeof(req));
	if (!target_name || !link_name) {
		errno = EFAULT;
		return(-1);
	}
	if (pvfs_checks_disabled) return symlink(target_name, link_name);
	/* Check to see if .pvfsdir exists in directory of link_name	*/
	if ((i = pvfs_detect(link_name, &fn, &saddr, &fs_ino, NULL, NOFOLLOW_LINK)) < 1) {
		if (i < 0) {
			PERROR("Error finding file");
			return(-1);
		}	
		return symlink(target_name, link_name);
	}
	len = snprintf(bothname, MAXPATHLEN, "%s", fn);
	if(len <= 0 || len >= MAXPATHLEN) {
		errno = EINVAL;
		PERROR("Could not append file names to string\n");
		return (-1);
	}
	bothlen += (len + 1);
	bothlen += snprintf(bothname + len + 1, MAXPATHLEN, "%s", target_name);
	/* Prepare request for file system  */
	req.dsize = bothlen;
	req.uid = getuid();
	req.gid = getgid();
	req.type = MGR_LINK;
	req.req.link.soft = 1;
	req.req.link.meta.fs_ino = fs_ino;
	req.req.link.meta.u_stat.st_uid = req.uid;
	req.req.link.meta.u_stat.st_gid = req.gid;
	req.req.link.meta.u_stat.st_mode = S_IFLNK | S_IRWXU | S_IRWXG | S_IRWXO;

	/* Send request to mgr */	
	if ((i = send_mreq_saddr(saddr, &req, bothname, &ack)) < 0) {
		PERROR("send_mreq_dmeta");
		return(-1);
	}
	return (0);
}

int pvfs_link(const char* target_name, const char *link_name)
{
	int i;
	mreq req;
	mack ack;
	struct sockaddr *saddr;
	char *fn = NULL;
	char bothname[MAXPATHLEN + MAXPATHLEN + 2];
	int len = 0, bothlen = 0;
	int64_t oldfs_ino, newfs_ino;

	memset(&req, 0, sizeof(req));
	if (!target_name || !link_name) {
		errno = EFAULT;
		return(-1);
	}
	if (pvfs_checks_disabled) return link(target_name, link_name);
	/* Check to see if .pvfsdir exists in directory of link_name and target_name */
	if ((i = pvfs_detect(link_name, &fn, &saddr, &newfs_ino, NULL, NOFOLLOW_LINK)) < 1) {
		if (i < 0) {
			PERROR("Error finding file");
			return(-1);
		}	
		return link(target_name, link_name);
	}
	len = snprintf(bothname, MAXPATHLEN, "%s", fn);
	if(len <= 0 || len >= MAXPATHLEN) {
		errno = EINVAL;
		PERROR("Could not append file names to string\n");
		return (-1);
	}
	bothlen += (len + 1);
	/* since link_name exists on a PVFS volume, target_name should also be on a PVFS volume */
	if ((i = pvfs_detect(target_name, &fn, &saddr, &oldfs_ino, NULL, NOFOLLOW_LINK)) < 1) {
		if (i < 0) {
			PERROR("Error finding file");
			return(-1);
		}	
		errno = EXDEV; /* cross file system */
		return (-1);
	}
	bothlen += snprintf(bothname + len + 1, MAXPATHLEN, "%s", fn);
	if(oldfs_ino != newfs_ino) {
		errno = EXDEV; /* cross file system */
		return (-1);
	}
	/* Prepare request for file system  */
	req.dsize = bothlen;
	req.uid = getuid();
	req.gid = getgid();
	req.type = MGR_LINK;
	req.req.link.soft = 0;
	req.req.link.meta.fs_ino = newfs_ino;
	req.req.link.meta.u_stat.st_uid = req.uid;
	req.req.link.meta.u_stat.st_gid = req.gid;
	req.req.link.meta.u_stat.st_mode = S_IFLNK | S_IRWXU | S_IRWXG | S_IRWXO;

	/* Send request to mgr */	
	if ((i = send_mreq_saddr(saddr, &req, bothname, &ack)) < 0) {
		PERROR("send_mreq_dmeta");
		return(-1);
	}
	return (0);
}

int pvfs_readlink(const char *path, char *buf, size_t bufsiz)
{
	int s, i, myeno;
	mreq req;
	mack ack;
	struct sockaddr *saddr;
	char *fn = NULL;
	int64_t fs_ino;

	memset(&req, 0, sizeof(req));
	if(!path || !buf) {
		errno = EFAULT;
		return -1;
	}
	if(bufsiz <= 0) {
		errno = EINVAL;
		return -1;
	}
	if (pvfs_checks_disabled) return readlink(path, buf, bufsiz);
	/* Check to see if .pvfsdir exists in directory of link_name and target_name */
	if ((i = pvfs_detect(path, &fn, &saddr, &fs_ino, NULL, NOFOLLOW_LINK)) < 1) {
		if (i < 0) {
			PERROR("Error finding file");
			return(-1);
		}	
		return readlink(path, buf, bufsiz);
	}
	/* Prepare request for file system  */
	req.dsize = strlen(fn);
	req.uid = getuid();
	req.gid = getgid();
	req.type = MGR_READLINK;
	/* Send request to mgr */	
	if ((s = send_mreq_saddr(saddr, &req, fn, &ack)) < 0) {
		PERROR("send_mreq_dmeta");
		return(-1);
	}
	if (ack.status) /* error */ {
		errno = ack.eno;
		return(-1);
	}
	if (!ack.dsize) /* null string */ {
		return(0);
	}
	if(bufsiz > ack.dsize) {
		bufsiz = ack.dsize;
	}
	if ((i = brecv(s, buf, bufsiz)) < 0) {
		myeno = errno;
		PERROR("readlink: brecv (getting data)");
		errno = myeno;
		return(-1);
	}
	return i;
}

/*
 * Local variables:
 *  c-indent-level: 3
 *  c-basic-offset: 3
 *  tab-width: 3
 *
 * vim: ts=3
 * End:
 */

==> pvfs-1.6.2.tgz: pvfs-1.6.2/lib/pvfs_symlink.c <==
/*
 * (C) 1995-2001 Clemson University and Argonne National Laboratory.
 *
 * See COPYING in top-level directory.
 */


/* This file contains PVFS library call for unlink.   	*/
/* It will determine if file is UNIX or PVFS and then 	*/
/* make the proper request.										*/

#include <lib.h>
#include <sys/param.h>
#include <meta.h>
#include <errno.h>
#include <sockio.h>

/* This file contains PVFS library call for symlink and hard links.   	*/
/* It will determine if file is UNIX or PVFS and then 						*/
/* make the proper request.															*/


extern int pvfs_checks_disabled;

int pvfs_symlink(const char* target_name, const char *link_name)
{
	int i;
	mreq req;
	mack ack;
	struct sockaddr *saddr;
	char *fn;
	int64_t fs_ino;
	char bothname[MAXPATHLEN + MAXPATHLEN + 2];
	int len = 0, bothlen = 0;

	memset(&req, 0, sizeof(req));
	if (!target_name || !link_name) {
		errno = EFAULT;
		return(-1);
	}
	if (pvfs_checks_disabled) return symlink(target_name, link_name);
	/* Check to see if .pvfsdir exists in directory of link_name	*/
	if ((i = pvfs_detect(link_name, &fn, &saddr, &fs_ino, NULL, NOFOLLOW_LINK)) < 1) {
		if (i < 0) {
			PERROR("Error finding file");
			return(-1);
		}	
		return symlink(target_name, link_name);
	}
	len = snprintf(bothname, MAXPATHLEN, "%s", fn);
	if(len <= 0 || len >= MAXPATHLEN) {
		errno = EINVAL;
		PERROR("Could not append file names to string\n");
		return (-1);
	}
	bothlen += (len + 1);
	bothlen += snprintf(bothname + len + 1, MAXPATHLEN, "%s", target_name);
	/* Prepare request for file system  */
	req.dsize = bothlen;
	req.uid = getuid();
	req.gid = getgid();
	req.type = MGR_LINK;
	req.req.link.soft = 1;
	req.req.link.meta.fs_ino = fs_ino;
	req.req.link.meta.u_stat.st_uid = req.uid;
	req.req.link.meta.u_stat.st_gid = req.gid;
	req.req.link.meta.u_stat.st_mode = S_IFLNK | S_IRWXU | S_IRWXG | S_IRWXO;

	/* Send request to mgr */	
	if ((i = send_mreq_saddr(saddr, &req, bothname, &ack, NULL)) < 0) {
		PERROR("send_mreq_dmeta");
		return(-1);
	}
	return (0);
}

int pvfs_link(const char* target_name, const char *link_name)
{
	int i;
	mreq req;
	mack ack;
	struct sockaddr *saddr;
	char *fn = NULL;
	char bothname[MAXPATHLEN + MAXPATHLEN + 2];
	int len = 0, bothlen = 0;
	int64_t oldfs_ino, newfs_ino;

	memset(&req, 0, sizeof(req));
	if (!target_name || !link_name) {
		errno = EFAULT;
		return(-1);
	}
	if (pvfs_checks_disabled) return link(target_name, link_name);
	/* Check to see if .pvfsdir exists in directory of link_name and target_name */
	if ((i = pvfs_detect(link_name, &fn, &saddr, &newfs_ino, NULL, NOFOLLOW_LINK)) < 1) {
		if (i < 0) {
			PERROR("Error finding file");
			return(-1);
		}	
		return link(target_name, link_name);
	}
	len = snprintf(bothname, MAXPATHLEN, "%s", fn);
	if(len <= 0 || len >= MAXPATHLEN) {
		errno = EINVAL;
		PERROR("Could not append file names to string\n");
		return (-1);
	}
	bothlen += (len + 1);
	/* since link_name exists on a PVFS volume, target_name should also be on a PVFS volume */
	if ((i = pvfs_detect(target_name, &fn, &saddr, &oldfs_ino, NULL, NOFOLLOW_LINK)) < 1) {
		if (i < 0) {
			PERROR("Error finding file");
			return(-1);
		}	
		errno = EXDEV; /* cross file system */
		return (-1);
	}
	bothlen += snprintf(bothname + len + 1, MAXPATHLEN, "%s", fn);
	if(oldfs_ino != newfs_ino) {
		errno = EXDEV; /* cross file system */
		return (-1);
	}
	/* Prepare request for file system  */
	req.dsize = bothlen;
	req.uid = getuid();
	req.gid = getgid();
	req.type = MGR_LINK;
	req.req.link.soft = 0;
	req.req.link.meta.fs_ino = newfs_ino;
	req.req.link.meta.u_stat.st_uid = req.uid;
	req.req.link.meta.u_stat.st_gid = req.gid;
	req.req.link.meta.u_stat.st_mode = S_IFLNK | S_IRWXU | S_IRWXG | S_IRWXO;

	/* Send request to mgr */	
	if ((i = send_mreq_saddr(saddr, &req, bothname, &ack, NULL)) < 0) {
		PERROR("send_mreq_dmeta");
		return(-1);
	}
	return (0);
}

int pvfs_readlink(const char *path, char *buf, size_t bufsiz)
{
	int s, i, myeno;
	mreq req;
	mack ack;
	struct sockaddr *saddr;
	char *fn = NULL;
	int64_t fs_ino;

	memset(&req, 0, sizeof(req));
	if(!path || !buf) {
		errno = EFAULT;
		return -1;
	}
	if(bufsiz <= 0) {
		errno = EINVAL;
		return -1;
	}
	if (pvfs_checks_disabled) return readlink(path, buf, bufsiz);
	/* Check to see if .pvfsdir exists in directory of link_name and target_name */
	if ((i = pvfs_detect(path, &fn, &saddr, &fs_ino, NULL, NOFOLLOW_LINK)) < 1) {
		if (i < 0) {
			PERROR("Error finding file");
			return(-1);
		}	
		return readlink(path, buf, bufsiz);
	}
	/* Prepare request for file system  */
	req.dsize = strlen(fn);
	req.uid = getuid();
	req.gid = getgid();
	req.type = MGR_READLINK;
	/* Send request to mgr */	
	if ((s = send_mreq_saddr(saddr, &req, fn, &ack, NULL)) < 0) {
		PERROR("send_mreq_dmeta");
		return(-1);
	}
	if (ack.status) /* error */ {
		errno = ack.eno;
		return(-1);
	}
	if (!ack.dsize) /* null string */ {
		return(0);
	}
	if(bufsiz > ack.dsize) {
		bufsiz = ack.dsize;
	}
	if ((i = brecv(s, buf, bufsiz)) < 0) {
		myeno = errno;
		PERROR("readlink: brecv (getting data)");
		errno = myeno;
		return(-1);
	}
	return i;
}

/*
 * Local variables:
 *  c-indent-level: 3
 *  c-basic-offset: 3
 *  tab-width: 3
 *
 * vim: ts=3
 * End:
 */
